<?php
/**
 * @package	com_contactenhanced
 * @copyright 2006-2020 IdealExtensions.com. All rights reserved.
 * @license	http://www.gnu.org/licenses/gpl.html GNU/GPL

 */

defined('_JEXEC') or die();

JLoader::register('ContactenchancedHelperRoute', JPATH_SITE . '/components/com_contactenhanced/helpers/route.php');

/**
 * Handles standard Joomla's Content contacts/categories
 *
 * This plugin is able to expand the categories keeping the right order of the
 * contacts acording to the menu settings and the user session data (user state).
 *
 * This is a very complex plugin, if you are trying to build your own plugin
 * for other component, I suggest you to take a look to another plugis as
 * they are usually most simple. ;)
 */

use Alledia\OSMap;
use Alledia\OSMap\Sitemap\Collector;
use Alledia\OSMap\Sitemap\Item;
use Joomla\Registry\Registry;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Uri\Uri;

class PlgOSMapContactenhanced extends OSMap\Plugin\Base implements OSMap\Plugin\ContentInterface
{
	/**
	 * @var PlgOSMapJoomla
	 */
	private static $instance = null;

	/**
	 * Returns the unique instance of the plugin
	 *
	 * @return PlgOSMapJoomla
	 */
	public static function getInstance()
	{
		if (empty(static::$instance)) {
			$dispatcher		= \JEventDispatcher::getInstance();
			static::$instance = new self($dispatcher);
		}

		return static::$instance;
	}

	/**
	 * Returns the element of the component which this plugin supports.
	 *
	 * @return string
	 */
	public function getComponentElement()
	{
		return 'com_contactenhanced';
	}

	/**
	 * This function is called before a menu item is used. We use it to set the
	 * proper uniqueid for the item
	 *
	 * @param Item	 $node	Menu item to be "prepared"
	 * @param Registry $params The extension params
	 *
	 * @return bool
	 * @throws Exception
	 */
	public static function prepareMenuItem($node, $params)
	{
		static::checkMemory();

		$db		= OSMap\Factory::getDbo();

		$linkQuery = parse_url($node->link);

		if (!isset($linkQuery['query'])) {
			return false;
		}

		$node->pluginParams = $params;

		parse_str(html_entity_decode($linkQuery['query']), $linkVars);

		$view = ArrayHelper::getValue($linkVars, 'view', '');
		$id	= ArrayHelper::getValue($linkVars, 'id', 0);

		switch ($view) {
			case 'archive':
				$node->adapterName = 'ContactEnhancedCategory';
				$node->uid		 = 'contactenhanced.archive.' . $node->id;
				$node->expandible	= true;

				break;

			case 'featured':
				$node->adapterName = 'ContactEnhancedCategory';
				$node->uid		 = 'contactenhanced.featured.' . $node->id;
				$node->expandible	= true;

				break;

			case 'categories':
			case 'category':
				$node->adapterName = 'ContactEnhancedCategory';
				$node->uid		 = 'contactenhanced.category.' . $id;
				$node->expandible	= true;

				break;

			case 'contact':
				$node->adapterName = 'ContactEnhancedContact';
				$node->expandible	= false;

				$paramAddImages	 = $params->get('add_images', 1);

				$query = $db->getQuery(true)
					->select(
						array(
							$db->quoteName('name', 'title'),
							$db->quoteName('created'),
							$db->quoteName('modified'),
							$db->quoteName('publish_up'),
							$db->quoteName('params'),
						)
					)
					->from($db->quoteName('#__ce_details'))
					->where($db->quoteName('id') . '=' . (int)$id);

				if ($paramAddImages) {
					$query->select(
						array(
							$db->quoteName('misc'),
							$db->quoteName('image'),
							$db->quoteName('category_image')
						)
					);
				}

				$db->setQuery($query);

				if (($item = $db->loadObject()) != null) {
					// Set the node UID
					$node->uid = 'contactenhanced.contact.' . $id;

					// Set dates
					$node->modified	= $item->modified;
					$node->created	= $item->created;
					$node->publishUp = $item->publish_up;


					if ($paramAddImages) {
						$node->images = self::getImages($item, $params);
					}
				} else {
					return false;
				}

				break;
		}

		return true;
	}
	
	private static function getImages($item,$params)
	{
		$container = OSMap\Factory::getContainer();
		$images = array();
		$maxImages = $params->get('max_images', 100);
		
		// Images
		if (!empty($item->image)) {
			$images[] = (object)array(
					'src'	=> $container->router->convertRelativeUriToFullUri($item->image),
					'title' => $item->title
			);
		}
		if (!empty($item->category_image)) {
			$images[] = (object)array(
					'src'	=> $container->router->convertRelativeUriToFullUri($item->category_image),
					'title' => $item->title
			);
		}
		// Images from text
		$images = array_merge(
				$images,
				(array)$container->imagesHelper->getImagesFromText($item->misc, $maxImages)
				);
		return $images;
	}

	/**
	 * Expands a com_contactenhanced menu item
	 *
	 * @param Collector $collector
	 * @param Item		$menuItem
	 * @param Registry	$params
	 *
	 * @return void
	 * @throws Exception
	 */
	public static function getTree($collector, $menuItem, $params)
	{

		

		$linkQuery = parse_url($menuItem->link);

		if (!isset($linkQuery['query'])) {
			return;
		}

		parse_str(html_entity_decode($linkQuery['query']), $linkVars);
		$view = ArrayHelper::getValue($linkVars, 'view', '');
		$id	= intval(ArrayHelper::getValue($linkVars, 'id', ''));

		/*
		 * Parameters Initialisation
		 */
		$paramExpandCategories = $params->get('expand_categories', 1) > 0;
		$paramExpandFeatured	= $params->get('expand_featured', 1);
		$paramIncludeArchived	= $params->get('include_archived', 2);


		$paramCatPriority	= $params->get('cat_priority', $menuItem->priority);
		$paramCatChangefreq = $params->get('cat_changefreq', $menuItem->changefreq);

		if ($paramCatPriority == '-1') {
			$paramCatPriority = $menuItem->priority;
		}
		if ($paramCatChangefreq == '-1') {
			$paramCatChangefreq = $menuItem->changefreq;
		}
		$params->set('cat_priority', $paramCatPriority);
		$params->set('cat_changefreq', $paramCatChangefreq);

		$paramArtPriority	= $params->get('contact_priority', $menuItem->priority);
		$paramArtChangefreq = $params->get('contact_changefreq', $menuItem->changefreq);

		if ($paramArtPriority == '-1') {
			$paramArtPriority = $menuItem->priority;
		}

		if ($paramArtChangefreq == '-1') {
			$paramArtChangefreq = $menuItem->changefreq;
		}

		$params->set('contact_priority', $paramArtPriority);
		$params->set('contact_changefreq', $paramArtChangefreq);

		

		switch ($view) {
			case 'category':
				if (empty($id)) {
					$id = intval($params->get('id', 0));
				}

				if ($paramExpandCategories && $id) {
					self::expandCategory($collector, $menuItem, $id, $params, $menuItem->id);
				}

				break;

			case 'featured':
				if ($paramExpandFeatured) {
					self::includeCategoryContent($collector, $menuItem, 'featured', $params, $menuItem->id);
				}

				break;

			case 'categories':
				if ($paramExpandCategories) {
					if (empty($id)) {
						$id = 1;
					}

					self::expandCategory($collector, $menuItem, $id, $params, $menuItem->id);
				}

				break;

			case 'archive':
				if ($paramIncludeArchived) {
					self::includeCategoryContent($collector, $menuItem, 'archived', $params, $menuItem->id);
				}

				break;

		}
	}

	/**
	 * Get all content items within a content category.
	 * Returns an array of all contained content items.
	 *
	 * @param Collector $collector
	 * @param Item		$parent the menu item
	 * @param int		$catid	the id of the category to be expanded
	 * @param Registry	$params parameters for this plugin on Xmap
	 * @param int		$itemid the itemid to use for this category's children
	 * @param int		$curlevel
	 *
	 * @return void
	 * @throws Exception
	 */
	protected static function expandCategory(
		$collector,
		$parent,
		$catid,
		$params,
		$itemid,
		$curlevel = 0
	) {
		static::checkMemory();

		$db = OSMap\Factory::getDBO();

		$where = array(
			'a.parent_id = ' . $catid,
			'a.published = 1',
			'a.extension=' . $db->quote('com_contactenhanced')
		);

		if (!$params->get('show_unauth', 0)) {
			$where[] = 'a.access IN (' . OSMap\Helper\General::getAuthorisedViewLevels() . ') ';
		}

		$query = $db->getQuery(true)
			->select(
				array(
					'a.id',
					'a.title',
					'a.alias',
					'a.access',
					'a.path AS route',
					'a.created_time AS created',
					'a.modified_time AS modified',
					'a.params',
					'a.metadata',
					'a.metakey'
				)
			)
			->from('#__categories AS a')
			->where($where)
			->order('a.lft');

		$items = $db->setQuery($query)->loadObjectList();

		$curlevel++;

		$node	 = null;
		$maxLevel = $parent->params->get('max_category_level', 100);

		if ($curlevel <= $maxLevel) {
			if (count($items) > 0) {
				$collector->changeLevel(1);

				foreach ($items as $item) {
					$node = (object)array(
						'id'						=> $item->id,
						'uid'						=> 'contactenhanced.category.' . $item->id,
						'browserNav'				=> $parent->browserNav,
						'priority'				 => $params->get('cat_priority'),
						'changefreq'				=> $params->get('cat_changefreq'),
						'name'					 => $item->title,
						'expandible'				=> true,
						'secure'					=> $parent->secure,
						'newsItem'				 => 0,
						'adapterName'				=> 'ContactEnhancedCategory',
						'pluginParams'			 => &$params,
						'parentIsVisibleForRobots' => $parent->visibleForRobots,
						'created'					=> $item->created,
						'modified'				 => $item->modified,
						'publishUp'				=> $item->created
					);

					// Keywords
					$paramKeywords = $params->get('keywords', 'metakey');
					$keywords		= null;
					if ($paramKeywords !== 'none') {
						$keywords = $item->metakey;
					}
					$node->keywords = $keywords;

					$node->slug	= $item->route ? ($item->id . ':' . $item->route) : $item->id;
					$node->link	= ContactenchancedHelperRoute::getCategoryRoute($node->slug);
					$node->itemid = $itemid;

					// Correct for an issue in Joomla core with occasional empty variables
					$linkUri = new Uri($node->link);
					$linkUri->setQuery(array_filter((array)$linkUri->getQuery(true)));
					$node->link = $linkUri->toString();

					if ($collector->printNode($node)) {
						self::expandCategory($collector, $parent, $item->id, $params, $node->itemid, $curlevel);
					}
				}

				$collector->changeLevel(-1);
			}
		}

		// Include Category's content
		self::includeCategoryContent($collector, $parent, $catid, $params, $itemid, $node);
	}

	/**
	 * Get all content items within a content category.
	 * Returns an array of all contained content items.
	 *
	 * @since 2.0
	 *
	 * @param Collector	$collector
	 * @param Item		$parent
	 * @param int|string $catid
	 * @param Registry	$params
	 * @param int		$itemid
	 * @param object	 $prevnode
	 *
	 * @return void
	 * @throws Exception
	 */
	public static function includeCategoryContent($collector, $parent, $catid, $params, $itemid, $prevnode = null)
	{
		static::checkMemory();

		$db		= OSMap\Factory::getDbo();

		if ($params->get('include_archived', 2)) {
			$where = array('(a.published = 1 or a.published = 2)');
		} else {
			$where = array('a.published = 1');
		}

		if ($catid == 'featured') {
			$where[] = 'a.featured=1';
		} elseif ($catid == 'archived') {
			$where = array('a.published=2');
		} elseif (is_numeric($catid)) {
			$where[] = 'a.catid=' . (int)$catid;
		}

		if (!$params->get('show_unauth', 0)) {
			$where[] = 'a.access IN (' . OSMap\Helper\General::getAuthorisedViewLevels() . ') ';
		}

		$nullDate = $db->quote($db->getNullDate());
		$nowDate	= $db->quote(OSMap\Factory::getDate()->toSql());

		$query = $db->getQuery(true)
			->select(
				array(
					'a.id',
					'a.name AS title',
					'a.alias',
					'a.catid',
					'a.created',
					'a.modified',
					'a.publish_up',
					'a.params',
					'a.metadata',
					'a.language',
					'a.metakey',
					'c.title AS categMetakey'
				)
			);

		if ($params->get('add_images', 1)) {
			$query->select(
				array(
					'a.misc',
					'a.image',
					'a.category_image'
				)
			);
		}

		$query
			->from('#__ce_details AS a')
			->join('LEFT', '#__categories AS c ON (a.catid = c.id)')
			->where($where)
			->where('(a.publish_up = ' . $nullDate . ' OR a.publish_up <= ' . $nowDate . ')')
			->where('(a.publish_down = ' . $nullDate . ' OR a.publish_down >= ' . $nowDate . ')');


		// Ordering
		$orderOptions	= array(
			'a.created',
			'a.modified',
			'a.publish_up',
			'a.hits',
			'a.name',
			'a.ordering'
		);
		$orderDirOptions = array(
			'ASC',
			'DESC'
		);

		$order	= ArrayHelper::getValue($orderOptions, $params->get('contact_order', 0), 0);
		$orderDir = ArrayHelper::getValue($orderDirOptions, $params->get('contact_orderdir', 0), 0);

		$orderBy = ' ' . $order . ' ' . $orderDir;
		$query->order($orderBy);

		$maxContacts = (int)$params->get('max_contacts');
		$db->setQuery($query, 0, $maxContacts);

		$items = $db->loadObjectList();
// 		echo ($query->__toString()); exit;
		if (count($items) > 0) {
			$collector->changeLevel(1);

			$paramExpandCategories = $params->get('expand_categories', 1);
			$paramExpandFeatured	= $params->get('expand_featured', 1);
			$paramIncludeArchived	= $params->get('include_archived', 2);

			foreach ($items as $item) {
				$node = (object)array(
					'id'					=> $item->id,
					'uid'					=> 'contactenhanced.contact.' . $item->id,
					'browserNav'			=> $parent->browserNav,
					'priority'				=> $params->get('contact_priority'),
					'changefreq'			=> $params->get('contact_changefreq'),
					'name'					=> $item->title,
					'created'				=> $item->created,
					'modified'				=> $item->modified,
					'publishUp'				=> $item->publish_up,
					'expandible'			=> false,
					'secure'				=> $parent->secure,
					'newsItem'				=> 1,
					'language'				=> $item->language,
					'adapterName'			=> 'ContactenhancedContact',
					'pluginParams'			=> &$params,
					'parentIsVisibleForRobots' => $parent->visibleForRobots
				);

				$keywords = array();

				$paramKeywords = $params->get('keywords', 'metakey');
				if ($paramKeywords !== 'none') {
					if (in_array($paramKeywords, array('metakey', 'both'))) {
						$keywords[] = $item->metakey;
					}

					if (in_array($paramKeywords, array('category', 'both'))) {
						$keywords[] = $item->categMetakey;
					}
				}
				$node->keywords = join(',', $keywords);

				$node->slug	= $item->alias ? ($item->id . ':' . $item->alias) : $item->id;
				$node->catslug = $item->catid;
				$node->link	= ContactenchancedHelperRoute::getContactRoute($node->slug, $node->catslug);

				// Set the visibility for XML or HTML sitemaps
				if ($catid == 'featured') {
					// Check if the item is visible in the XML or HTML sitemaps
					$node->visibleForXML	= in_array($paramExpandFeatured, array(1, 2));
					$node->visibleForHTML	= in_array($paramExpandFeatured, array(1, 3));
				} elseif ($catid == 'archived') {
					// Check if the item is visible in the XML or HTML sitemaps
					$node->visibleForXML	= in_array($paramIncludeArchived, array(1, 2));
					$node->visibleForHTML	= in_array($paramIncludeArchived, array(1, 3));
				} elseif (is_numeric($catid)) {
					// Check if the item is visible in the XML or HTML sitemaps
					$node->visibleForXML	= in_array($paramExpandCategories, array(1, 2));
					$node->visibleForHTML	= in_array($paramExpandCategories, array(1, 3));
				}

				

				if ($params->get('add_images', 1)) {
					$node->images = self::getImages($item, $params);
				}

				

				if ($collector->printNode($node) && $node->expandible) {
					self::printSubNodes($collector, $parent, $params, $node->subnodes, $node);
				}
			}

			$collector->changeLevel(-1);
		}
	}

	/**
	 * @param Collector $collector
	 * @param Item		$parent
	 * @param Registry	$params
	 * @param array	 $subnodes
	 * @param object	$item
	 *
	 * @return void
	 * @throws Exception
	 */
	protected static function printSubNodes($collector, $parent, $params, $subnodes, $item)
	{
		static::checkMemory();

		$collector->changeLevel(1);

		$i = 0;
		foreach ($subnodes as $subnode) {
			$i++;

			$subnode->browserNav= $parent->browserNav;
			$subnode->priority	= $params->get('contact_priority');
			$subnode->changefreq= $params->get('contact_changefreq');
			$subnode->secure	= $parent->secure;
			$subnode->created	= $item->created;
			$subnode->modified	= $item->modified;
			$subnode->publishUp	= isset($item->publish_up) ? $item->publish_up : $item->created;

			$collector->printNode($subnode);

			$subnode = null;
			unset($subnode);
		}

		$collector->changeLevel(-1);
	}
}
